/*
** Program:
**	RegForm
**
** Purpose:
**	Provides a simple registration/comment form to facilitate
**	end-user software registration and comment/bug reporting.
**	A file '~/.regform' is created if the user actually
**	attempts to send the form to the author, thus creating a
**	kind of registration history.  If the file already exists,
**	the current application name is added to the history.  This
**	provides a good mechanism for applications to check if
**	users have registered or not.
**
** Usage:
**	regform <flags>
**
**	-a <app name>		: Application name (default below)
**	-c			: Comments/Bugs form
**	-r			: Registration form (default)
**	-s <sendto address>	: E-mail address to send to (default below)
**	-v <app version>	: Application version (default below)
**
** Notes:
**	The default values for all fields can be set by modifying the
**	variable 'regform' below.
**
**-------------------------------------------------------------------------
**
**                    Copyright (c) 1994 James R. Dyvig
** 
** Permission to use, copy, modify, and distribute this software and its
** documentation for educational, research, and non-profit purposes
** without fee is hereby granted, provided that the above copyright notice
** appear in all copies and that both that copyright notice and this
** permission notice appear in supporting documentation.
** 
** Permission to incorporate this software into commercial products can be
** obtained with the written consent of the author.
** 
** This software is provided AS IS with no warranties of any kind.  The
** author shall have no liability with respect to the infringement of
** copyrights, trade secrets or any patents by this file or any part
** thereof.  In no event will the author be liable for any lost revenue or
** profits or other special, indirect and consequential damages.
**
**-------------------------------------------------------------------------
*/
#include <stdio.h>
#include <Xm/Form.h>
#include "regform.h"

RegForm	regform = {	NULL,			/* Name */
			NULL,			/* Company */
			NULL,			/* Address */
			NULL,			/* City/State/Zip */
			NULL,			/* Country */
			NULL,			/* E-mail */
			NULL,			/* Machines */
			NULL,			/* Applications */
			NULL,			/* GUI Builders */
			NULL,			/* Comments */
			"jdyvig@photon.com",	/* Sent To Address */
			REGFORM_NAME,		/* Application Name */
			REGFORM_VERSION};	/* Application Version */

String	fallbacks[] = {
	"*foreground:                   white",
	"*background:                   cyan4",
	"*textfield*marginHeight:       2",
	"*fontList: -*-helvetica-bold-r-normal-*-14-*-*-*-*-*-*-*",
	"*grouplabel*fontList: -*-helvetica-bold-o-normal-*-14-*-*-*-*-*-*-*",
	"*titlelabel*fontList: -*-helvetica-bold-r-normal-*-18-*-*-*-*-*-*-*",
	"*titlelabel*foreground:	yellow",
	NULL};

Boolean	use_registration = True;

void	print_help(void);

int main(int argc, char **argv)
{
	int		ac;		/* Argument count */
	Arg		al[30];		/* Argument list */
	XtAppContext	appContext;	/* Application Context for topLevel */
	Widget		form;		/* Background form to place menu */
	Widget		menuBar;	/* Menu bar */
	Widget		topLevel;	/* Base widget for app */
	Widget		workArea;
	extern void 	create_input_form(Widget workArea);
	extern Widget	main_menubar(Widget form);
	extern void	parse_command_line(int argc, char **argv);
/*
** Parse command line
*/
	parse_command_line(argc, argv);
/*
** Create the top level shell and set the title and its size
*/
	topLevel = XtAppInitialize(&appContext, REGFORM_NAME, NULL, 0,
		(Cardinal *) &argc, argv, fallbacks, NULL, 0);
/*
** Set attributes of the top level shell.  We set the height to simulate
** possible other widgets in the GUI.  The width is automatically adjusted
** to the width of the menu buttons.
*/
	ac = 0;
	XtSetArg(al[ac], XmNtitle,	REGFORM_NAME); ac++;
	XtSetValues(topLevel, al, ac);
/*
** Create a form widget to put the menu bar on
*/
	ac = 0;
	form = XmCreateForm(topLevel, "form", al, ac);
	XtManageChild(form);

	menuBar = main_menubar(form);
/*
** Create work area
*/
	ac = 0;
	XtSetArg(al[ac], XmNtopAttachment,	XmATTACH_WIDGET); ac++;
	XtSetArg(al[ac], XmNtopWidget,		menuBar); ac++;
	XtSetArg(al[ac], XmNbottomAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNleftAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightAttachment,	XmATTACH_FORM); ac++;

	workArea = XmCreateForm(form, "form", al, ac);
	XtManageChild(workArea);
/*
** Create input form
*/
	create_input_form(workArea);
/*
** Realize the top level widget and enter the event loop
*/
	XtRealizeWidget(topLevel);

	XtAppMainLoop(appContext);
}


/*-----------------------------------------------------------------
** Function: 	parse_command_line
**
*/

void parse_command_line(int argc, char **argv)
{
	int	i;

	for (i=1;i<argc;i++) {
		if (argv[i][0] == '-') {
			switch(argv[i][1]) {
			case 'a':
				if (i+1 >= argc || argv[i+1][0] == '-') {
					fprintf(stderr, "Missing app name\n\n");
					print_help();
					exit(-1);
				}

				regform.codename = argv[i+1];
				i++;

				break;
			case 'c':
				use_registration = False;
				break;
			case 'r':
				use_registration = True;
				break;
			case 's':
				if (i+1 >= argc || argv[i+1][0] == '-') {
					fprintf(stderr, "Missing address\n\n");
					print_help();
					exit(-1);
				}

				regform.sendto = argv[i+1];
				i++;

				break;
			case 'v':
				if (i+1 >= argc || argv[i+1][0] == '-') {
					fprintf(stderr, "Missing version\n\n");
					print_help();
					exit(-1);
				}

				regform.version = argv[i+1];
				i++;

				break;
			default:
				fprintf(stderr,"Unknown flag %s\n",argv[i]);
				break;
			}
		}
	}
}


/*-----------------------------------------------------------------
** Function: 	print_help
**
*/

void print_help(void)
{
	printf("             ----- %s %s -----\n",
		REGFORM_NAME, REGFORM_VERSION);
	printf("\n");
	printf("-a <app name>		: Application name, no spaces\n");
	printf("-c			: Comments/Bugs form\n");
	printf("-r			: Registration form (default)\n");
	printf("-s <sendto address>	: E-mail address to send to\n");
	printf("-v <app version>	: Application version, no spaces\n");
	printf("\n");
}
