/*-------------------------------------------------------------------------
**
**                    Copyright (c) 1994 James R. Dyvig
** 
** Permission to use, copy, modify, and distribute this software and its
** documentation for educational, research, and non-profit purposes
** without fee is hereby granted, provided that the above copyright notice
** appear in all copies and that both that copyright notice and this
** permission notice appear in supporting documentation.
** 
** Permission to incorporate this software into commercial products can be
** obtained with the written consent of the author.
** 
** This software is provided AS IS with no warranties of any kind.  The
** author shall have no liability with respect to the infringement of
** copyrights, trade secrets or any patents by this file or any part
** thereof.  In no event will the author be liable for any lost revenue or
** profits or other special, indirect and consequential damages.
**
**-------------------------------------------------------------------------
*/

#include <stdio.h>
#include <malloc.h>
#include <stdlib.h>

#include "list.h"

/*------------------------------------------------------------------------------
 * function:	 make_list
 * description:	 creates an empty list
 * parameters:	 size - size of list data elements
 * returns:	 ptr to a list head; or NULL if can't allocate space
 * side effects: empty list is created if enough space
 *----------------------------------------------------------------------------*/

list* make_list(int size)
{
	list*	temp;
	
	if ((temp = (list*) malloc(sizeof(list))) != NULL) {
		temp->head = NULL;
		temp->current = NULL;
		temp->tail = NULL;
		temp->size = size;
	}
	else
		fprintf(stderr, "make_list: not enough space to create list\n");
	return temp;
}

/*------------------------------------------------------------------------------
 * function:	 add
 * description:	 adds an element immediately after the current element
 * parameters:	 l - list in which to add
 *		 element - element to add
 * returns:	 1 if add ok; 0 if not enough space
 * side effects: element is added to list if enough space
 *----------------------------------------------------------------------------*/

int add(list *l, void *element)
{
	lnode*	temp;

	if (l == NULL) {
		fprintf(stderr, "add: list not initialized\n");
		return 0;
	}
	if ((temp = (lnode*) malloc(sizeof(lnode))) == NULL) goto error;
	if (l->size == 0) {
		temp->data = (void *) element;
	} else {
		if ((temp->data = (void*) malloc(l->size)) == NULL) {
			free(temp);
			goto error;
		}
		bcopy(element, temp->data, l->size);
	}
/*
** Case 1:  No elements in list
*/
	if (l->tail == NULL) {
		l->head = temp;
		l->tail = temp;
		temp->prev = NULL;
		temp->next = NULL;
/*
** Case 2:  At end of list
*/
	} else if (l->current == l->tail) {
		temp->next = NULL;
		temp->prev = l->current;

		l->current->next = temp;

		l->tail = temp;
/*
** Case 3:  Normal...
*/
	} else {
		l->current->next->prev = temp;

		temp->next = l->current->next;
		temp->prev = l->current;

		l->current->next = temp;
	}

	l->current = temp;

	return 1;

error:	fprintf(stderr, "add: not enough space to insert\n");
	return 0;
}


/*------------------------------------------------------------------------------
 * function:	 insert
 * description:	 inserts an element at the tail of a list
 * parameters:	 l - list in which to insert
 *		 element - element to insert
 * returns:	 1 if insert ok; 0 if not enough space
 * side effects: element is added to list if enough space
 *----------------------------------------------------------------------------*/

int insert(list *l, void *element)
{
	lnode*	temp;

	if (l == NULL) {
		fprintf(stderr, "insert: list not initialized\n");
		return 0;
	}
	if ((temp = (lnode*) malloc(sizeof(lnode))) == NULL) goto error;
	if (l->size == 0) {
		temp->data = (void *) element;
	} else {
		if ((temp->data = (void*) malloc(l->size)) == NULL) {
			free(temp);
			goto error;
		}
		bcopy(element, temp->data, l->size);
	}
/*
** Case 1:  No elements
*/
	if (l->tail == NULL) {
		l->head = temp;
		l->tail = temp;
		temp->prev = NULL;
		temp->next = NULL;
/*
** Case 2:  At head
*/
	} else if (l->current == l->head) {
		temp->prev = NULL;
		temp->next = l->current;
		l->current->prev = temp;
		l->head = temp;
/*
** Case 3:  Normal...
*/
	} else {
		l->current->prev->next = temp;
		temp->prev = l->current->prev;
		temp->next = l->current;
		l->current->prev = temp;
	}

	l->current = temp;
	return 1;

error:	fprintf(stderr, "insert: not enough space to insert\n");
	return 0;
}

/*------------------------------------------------------------------------------
 * function:	 delete
 * description:	 delets an element at the tail of a list
 * parameters:	 l - list in which to insert
 *		 element - element to insert
 * returns:	 1 if insert ok; 0 if not enough space
 * side effects: element is added to list if enough space
 *----------------------------------------------------------------------------*/

int delete(list *l)
{
	lnode*	temp;

	if (l == NULL) {
		fprintf(stderr, "insert: list not initialized\n");
		return 0;
	}
/*
** Case 1:  No elements
*/
	if (l->tail == NULL) {
		return 0;
/*
** Case 2:  At head, one element
*/
	} else if (l->current == l->head && l->current == l->tail) {
		if (l->current->data)
			free(l->current->data);

		free(l->current);

		l->head = NULL;
		l->current = NULL;
		l->tail = NULL;
/*
** Case 3:  At head
*/
	} else if (l->current == l->head) {
		l->head = l->head->next;
		l->head->prev = NULL;

		/*
		if (l->current->data)
			free(l->current->data);
		*/
		free(l->current);

		l->current = l->head;
/*
** Case 4:  At tail
*/
	} else if (l->current == l->tail) {
		l->tail = l->tail->prev;
		l->tail->next = NULL;

		/*
		if (l->current->data)
			free(l->current->data);
		*/
		free(l->current);

		l->current = l->tail;
/*
** Case 5:  Normal...
*/
	} else {
		temp = l->current->next;

		l->current->prev->next = l->current->next;
		l->current->next->prev = l->current->prev;

		/*
		if (l->current->data)
			free(l->current->data);
		*/
		free(l->current);

		l->current = temp;
	}

	return 1;
}

/*------------------------------------------------------------------------------
 * function:	 current
 * description:	 gets the first element in a list
 * parameters:	 l - list from which to return first element
 * returns:	 ptr to first element if ok; NULL if empty list
 * side effects: alters current field of list (affects next call to next)
 *----------------------------------------------------------------------------*/

void* current(list *l)
{
	if (l->current) {
		return l->current->data;
	} else {
		return NULL;
	}
}

/*------------------------------------------------------------------------------
 * function:	 first
 * description:	 gets the first element in a list
 * parameters:	 l - list from which to return first element
 * returns:	 ptr to first element if ok; NULL if empty list
 * side effects: alters current field of list (affects next call to next)
 *----------------------------------------------------------------------------*/

void* first(list *l)
{
	if ((l == NULL) || (l->head == NULL)) return NULL;
	l->current = l->head;
	return l->head->data;
}

/*------------------------------------------------------------------------------
 * function:	 last
 * description:	 gets the last element in a list
 * parameters:	 l - list from which to return last element
 * returns:	 ptr to last element if ok; NULL if empty list
 * side effects: alters current field of list (affects next call to prev)
 *----------------------------------------------------------------------------*/

void* last(list *l)
{
	if ((l == NULL) || (l->tail == NULL)) return NULL;
	l->current = l->tail;
	return l->tail->data;
}

/*------------------------------------------------------------------------------
 * function:	 next
 * description:	 gets the next element in a list
 * parameters:	 l - list from which to get next element
 * returns:	 ptr to next element if ok; NULL if end of list
 * side effects: alters current field of list (affects next call to next)
 *----------------------------------------------------------------------------*/

void* next(list *l)
{
	if ((l == NULL) || (l->current->next == NULL)) return NULL;
	l->current = l->current->next;
	return l->current->data;
}

/*------------------------------------------------------------------------------
 * function:	 prev
 * description:	 gets the previous element in a list
 * parameters:	 l - list from which to get previous element
 * returns:	 ptr to previous element if ok; NULL if front of list
 * side effects: alters current field of list (affects next call to prev)
 *----------------------------------------------------------------------------*/

void* prev(list *l)
{
	void*	temp;

	if ((l == NULL) || (l->current->prev == NULL)) return NULL;
	l->current = l->current->prev;
	return l->current->data;
}

/*------------------------------------------------------------------------------
 * function:	 set_current
 * description:	 gets the first element in a list
 * parameters:	 l - list from which to return first element
 * returns:	 ptr to first element if ok; NULL if empty list
 * side effects: alters current field of list (affects next call to next)
 *----------------------------------------------------------------------------*/

int set_current(list *l, void *element)
{
	lnode	*temp;
	if (l == NULL) return 0;

	temp = l->head;
	if (temp == NULL) return 0;

	do {
		if (temp->data == element) {
			l->current = temp;
			return 1;
		}
	} while ((temp = temp->next) != NULL);

	return 0;
}

/*------------------------------------------------------------------------------
 * function:	 free_list
 * description:	 frees the memory allocated to a list
 * parameters:	 l - list to free
 * returns:	 void
 * side effects: list is freed if it exists
 *----------------------------------------------------------------------------*/

void free_list(list *l)
{
	if (l != NULL) {
		while (l->head != NULL) {
			l->current = l->head->next;
			if (l->size != 0) {
				free(l->head->data);
			}
			free(l->head);
			l->head = l->current;
		}
		free(l);
	}
}

/*------------------------------------------------------------------------------
 * function:	 listerror
 * description:	 gives a list error message and exit
 * parameters:	 none
 * returns:	 void
 * side effects: exits
 *----------------------------------------------------------------------------*/

void listerror()
{
	fprintf(stderr,"list error\n");
	exit(-1);
}
